/* File: bisectmethod.c 
   Find a root for projectile funct using bisection method in range [60, 70] */
#include <stdio.h>
#include <math.h>        /* for fabs() */
#include <float.h>       /* for FLT_EPSILON */
#define M_G   9.81       /* gravitational acceleration constant g */

/* function for the y-coordinate of the projectifle */
double func(double x) {
   double theta0,        /* initial velocity */
          v0,            /* projection angle */
          y;             /* y coordinate */
   theta0 = 15*M_PI/180; /* set the projection angle in radian */
   v0 = 35.0;            /* set the initial velocity */
   y = tan(theta0)*x - M_G*x*x/(2*v0*v0*cos(theta0)*cos(theta0));
   return y;
}

int main() {
    double xleft, fleft;       /* x and f(x) on the left of the root */
    double xright;             /* x on the right of the root */
    double xmid, fmid;         /* x and f(x) on the mid of xleft and xright */
    double tolerance;          /* convergence tolerance */
    double x0 = 60.0, xf = 70.0; /* initial end points */

    /* initialize xleft and xright with initial end points */
    xleft  = x0;   
    xright = xf;
    /* use machine epsilon for float type as convergence tolerance */
    tolerance=FLT_EPSILON;

    /* The signs of f(xleft) and f(xright) must be different 
       to guarantee there is a root */
    fleft = func(xleft);            /* calculate f(xleft) */
    if(fleft * func(xright) > 0.0) {    /* invalid x0 and xf */
       printf("Error: invalid end points\n");
       return -1;          /* return -1 for invalid xleft and xright */
    }

    xmid = (xleft+xright)/2.0;      /* mid point */
    fmid = func(xmid);              /* f() in the mid point */
    while(fabs(fmid) > tolerance) {/* convergence criterion to terminate loop */
        /* f(xleft)*f(mid) to determine the updating of left or right point */
        if(fleft*fmid <= 0.0) {     /* mid point becomes the right point */
            xright = xmid;          /* update the right point */
        }
        else {                      /* mid point becomes the left point */
            xleft = xmid;           /* update the left point */
            fleft = fmid;           /* update f(xleft) */
        }
        xmid = (xleft+xright)/2.0;  /* update the mid point */
        fmid = func(xmid);          /* f() in the mid point */
    } 
    printf("x = %f m, func(x) = %g\n", xmid, fmid);
    return 0;   
}
