/* File: acceldata2DCurve.cpp 
   Plot accelereration versus time with 11 and 100 points on the same figure */ 
#include <stdio.h>
#include <math.h>       /* for sin() */ 
#include <chplot.h>     /* for class CPlot for plotting */

#define M_G   9.81      /* gravitational acceleration constant g */
#define N1    11        /* num of points for the 1st curve */
#define N2    100       /* num of points for the 2nd curve */

int main() {
    /* declare two sets of arrays for plottting for a versus t */
    double a1[N1], t1[N1], a2[N2], t2[N2];
    double t0, tf, mu, m, p;
    int i;
    CPlot plot;         /* instantiate a plotting class */

    /* set mu, mass, initial time, and final time */
    mu = 0.2;
    m = 5.0;
    t0 = 0.0;
    tf = 10.0;

    /* calculate t1[i] and a1[i] with N1 points */
    for(i=0; i<N1; i++) {
        t1[i] = t0+i*(tf - t0)/(N1-1);
        p = 4*sin(t1[i]-3)+20;
        a1[i] = (p-mu*m*M_G)/m;
    }
    /* calculate t2[i] and a2[i] with N2 points */
    for(i=0; i<N2; i++) {
        t2[i] = t0+i*(tf - t0)/(N2-1);
        p = 4*sin(t2[i]-3)+20;
        a2[i] = (p-mu*m*M_G)/m;
    }

    /* use a plotting class plot two curves on the same figure */
    /* set title of the plot */
    plot.title("Acceleration Plot");
    /* set the label on the x-axis */
    plot.label(PLOT_AXIS_X, "time (s)");
    /* set the label on the y-axis */
    plot.label(PLOT_AXIS_Y, "accel (m/s^2)");
    /* add data a1[] and t1[] for plotting in the 1st curve */
    plot.data2DCurve(t1, a1, N1);
    /* set the legend for the 1st curve */
    plot.legend("11 points", 0);
    /* add data a2[] and t2[] for plotting in the 2nd curve */
    plot.data2DCurve(t2, a2, N2);
    /* set the legend for the 2nd curve */
    plot.legend("100 points", 1);
    /* after the above setup, ready and go, plot and display it! */
    plot.plotting();
    return 0;
}
