/* File: linearsys.c
   Solve a system of linear equations Ax=b for x using CLAPACK */
#include <stdio.h>
#include <clapack.h>   /* definitions for CLAPACK */

/*
int dgesv_(int *n, int *nrhs, double *a, int *lda, int *ipiv, 
           double *b, int *ldb, int *info);
*/

/* define the size of matrix and vector */
#define N 3    

int main() {
    /* declare and initialize 3x3 matrix A and vector b */
    double a[N][N] = {3, 5, 6, 
                      4, 2, 1,
                      0, 7, 1};
    double at[N][N];          /* transpose of matrix A */
    double b[N] = {-1, 8, 5}; /* original vector b and solution x */

    int n = N;     /* number of columns of matrix A  */
    int nrhs = 1;  /* number of columns of b, usually 1  */
    int lda = N;   /* number of rows (Leading dimension of A) of A  */
    int ipiv[N];   /* pivot indices  */
    int ldb = N;   /* number of rows of b  */
    int info;      /* status of the calling the function */
    int i, j;      /* array indices */
 
    /* obtain transpose of matrix A */
    for(i=0; i<N; i++) {
        for(j=0; j<N; j++) {
             at[j][i] = a[i][j];
        }
    }
    
    /* solve for x in Ax=b  using function dgesv_() in CLAPACK */
    dgesv_(&n, &nrhs, &at[0][0], &lda, ipiv, b, &ldb, &info);

    if(info == 0) /* succeed */
       printf("x = %g, %g, %g\n", b[0], b[1], b[2]); /* print vector x */
    else
       printf("dgesv_() failed %d\n", info);
    return 0;
}
