/* File: linreg.cpp
   Perform linear regression analysis for a set of data.
   Plot the fitted line in comparison with the original scattered data points */
#include <stdio.h>
#include <chplot.h>   /* for class CPlot for plotting */
#include "statlib.h"  /* for mean()  in the statistical lib */
#define NUM   100     /* number of points for plotting */
double b, v;          /* linear regression coefficients */

/* linear function y = b + v*t */
double func(double t) {
    double y;
    y = b + v*t;
    return y;
}
int main() {
    /* declare and initialize the original data */
    double t[] = {0.0, 3.0, 5.0, 8.0,  10.0, 12.0, 15.0, 18.0};
    double y[] = {2.0, 5.2, 6.0, 10.0, 10.4, 13.4, 14.8, 18.0};
    /* mean values and sums in the formulas for regression analysis */
    double t_mean, y_mean, titm, sumty=0, sumts=0;
    int i, n, num=0; /* index, number of data points, data set number */
    CPlot plot;      /* instantiate a plotting class */

    n = sizeof(t)/sizeof(double); /* calculate the number of data points */
    /* calculate the linear regression coefficients based on the formulas */
    t_mean = mean(n, t);
    y_mean = mean(n, y);
    for(i=0; i<n; i++) {
       titm = t[i] - t_mean;
       sumty += titm*(y[i]-y_mean);
       sumts += titm*titm;
    }
    v = sumty/sumts;
    b = y_mean - v*t_mean;
    /* display the linear regression coefficients v and b, and find y(11.0) */
    printf("v = %.2f\n", v);
    printf("b = %.2f\n", b);
    printf("y is = %.2f (m) at time t = 11 (s)\n", func(11.0));

    /* set the label on the x-axis */
    plot.label(PLOT_AXIS_X, "t (seconds)");
    /* set the label on the y-axis */
    plot.label(PLOT_AXIS_Y, "y (meters)");
    /* set the range for x-axis from 0 to 18 */ 
    plot.axisRange(PLOT_AXIS_Y, 0.0, 18.0);
    /* add data t[] and y[] as the 1st set of data for plotting */
    plot.data2DCurve(t, y, n);
    /* set the scattered point plot type for the 1st set of data */
    plot.plotType(PLOT_PLOTTYPE_POINTS, num);
    /* set function func2D() plotted from t[0] to t[n-1] with NUM points */
    plot.func2D(t[0], t[n-1], NUM, func);
    /* after the above setup, ready and go, plot and display it! */
        plot.nativeCmd("unset mouse");
    plot.plotting();
    return 0;
}
