/* File: sortarray2d.c (run in C99 and Ch only)
   Compile in gcc, use command 'gcc -std=c99 sortarray2d.c' 
   Sort two-dimensional arrays of different sizes in row-wise ascending 
   order by the same function with arguments of variable length arrays */
#include <stdio.h>

/* extents for arrays a1 and a2 in function main() */
#define M1 2
#define N1 3
#define M2 3
#define N2 4

/* function to sort array a in row-wise ascending order using selection sort.
   argument medians[] contains the medians for each row */
void sortArray2d(int m, int n, double a[m][n], double medians[m]) { 
    int i, j, k, imin; /* array indices */
    double temp;       /* temporary variable for swap */
    
    /* outer loop for each row of array a */
    for(k = 0; k < m; k++) {
        /* sort each row based on the column index of array a */
        for(i = 0; i < n-1; i++) {
            imin = i;
            /* sort one row at a time, the row index k for array a is fixed 
               inside this i-loop */
            for(j = i+1; j < n; j++) {
                if(a[k][imin] > a[k][j]) { 
                    imin = j;       /* save the index for the new minimum */
                }
            }
            /* swap the values for elements a[k][i] and a[k][imin] */
            if(imin != i) {
                temp = a[k][i];
                a[k][i] = a[k][imin];
                a[k][imin] = temp;
            }
        }
        /* calculate the medians for the sorted row */
        if(n%2) /* the total number of elements is odd */
           medians[k] = a[k][n/2];
        else    /* the total number of elements is even */
           medians[k] = (a[k][n/2-1] + a[k][n/2])/2.0;
    }
}

/* function to print elements of a two-dimensional array */
void printArray2d(int m, int n, double a[m][n]) {
    int i, j;
    for(i = 0; i < m; i++) {
        for(j = 0; j < n; j++) {
            printf("%4g", a[i][j]);
        }
        printf("\n");
    }
}

/* function to print elements of a one-dimensional array */
void printArray(int n, double a[n]) {
    int i;
    for(i = 0; i < n; i++) {
        printf("%4g", a[i]);
    }
    printf("\n");
}

int main() { 
    /* declare two arrays a1 and a2 with different sizes */
    double a1[M1][N1] = {3.0, 21.0, -3.0,
                         8.0, -9.0, 10.0};
    double a2[M2][N2] = {3.0,   21.0,  0.0, -3.0,
                         34.0, -14.0, 45.0, 18.0,
                         8.0,   -9.0,  4.0, 10.0};
    double median1[M1], median2[M2]; /* medians for arrays a1 and a2 */
   
    /* display the original and sorted data for array a1 */
    printf("The original data:\n");
    printArray2d(M1, N1, a1);
    sortArray2d(M1, N1, a1, median1); 
    printf("The sorted data in row-wise ascending order:\n");
    printArray2d(M1, N1, a1);
    printf("The medians in each row:\n");
    printArray(M1, median1);

    /* display the original and sorted data for array a2 */
    printf("\nThe original data:\n");
    printArray2d(M2, N2, a2);
    sortArray2d(M2, N2, a2, median2);
    printf("The sorted data in row-wise ascending order:\n");
    printArray2d(M2, N2, a2);
    printf("The medians in each row:\n");
    printArray(M2, median2);
    return 0;
}
