/* File: acceldynmemory.c
   Calculate acceleration, force, and time using arrays. 
   The memory for array for force is dynamically allocated. */
#include <stdio.h>
#include <math.h>    /* for sin() */

#define M_G   9.81   /* gravitational acceleration constant g */
#define N1    11     /* number of points for a, p, and t */
#define N2     6     /* number of points for a, p, and t */

/* pass arrays t and a as pointers */
int accel(int n, double *t, double *a) {
    double *p;       /* declare p as a pointer */
    double mu, m, t0, tf;
    int i;

    /* set mu, mass, initial time, and final time */
    mu = 0.2;
    m = 5.0;
    t0 = 0;
    tf = 10.0;

    /* allocate memory for array p */
    p = (double *) malloc(sizeof(double)*n);
    if(p == NULL) {
       printf("Error: not enough memory\n");
       return -1;
    }

    /* calculate t[i], pi[i], a[i] */
    for(i=0; i<n; i++) {
        t[i] = t0+i*(tf - t0)/(n-1);
        p[i] = 4*sin(t[i]-3)+20;
        a[i] = (p[i]-mu*m*M_G)/m;
    }

    /* post processing of the data by printing out or plotting.
       Here, data for t, p, and a are printed out */
    printf(" time (s) force (N) accel (m/s^2)\n");
    printf("---------------------------------\n");
    for(i=0; i<n; i++) {
        printf("%8.4f  %8.4f  %8.4f\n", t[i], p[i], a[i]);
    }
    printf("\n");  /* a newline to separate each function call */
    free(p);       /* free memory allocated for p */
    return 0;
}

int main() {
    double t1[N1], a1[N1]; /* declare array t1 and a1 with N1 points */
    double t2[N2], a2[N2]; /* declare array t2 and a2 with N2 points */

    accel(N1, t1, a1);     /* pass arrays with N1 points */
    accel(N2, t2, a2);     /* pass arrays with N2 points */
    return 0;
}
