/* File: accelpassfun.c
   Calculate accelerations by passing a function to 
   a pointer to functions for different formulas for forces */
#include <stdio.h>
#include <chplot.h>

#define M_G   9.81         /* gravitational acceleration constant g */
#define N 200              /* num of elements for plotting */

/* 1st function for force */
double force1(double t) {
    double p;

    p = 4*cos(t-3);
    return p;
}

/* 2nd function for force */
double force2(double t) {
    double p;

    p = 4*(t*t-3);
    return p;
}

/* function to calculate acceleration, 
   the formula for forces is passed as the second argument */
void accel(int n, double (*force)(double t)) {
    /* declare variables for accel, time, t0 and tf, mu, m, p */
    double a[n], t[n], t0, tf;
    double mu, m, p;
    int i;

    /* set mu, mass, initial time, and final time */
    mu = 0.2;
    m = 5.0;
    t0 = 0.0;
    tf = 10.0;
    for(i=0; i<n; i++) {
        t[i] = t0+i*(tf - t0)/(n-1);  /* calculate t[i] */
        /* calculate force using pointer to function */
        p = force(t[i]);
        a[i] = (p-mu*m*M_G)/m;        /* calculate a[i] */
    }
    /* plot acceleration versus time */
    plotxy(t, a, "Acceleration Plot", "time (s)", "accel (m/s^2)");
}
int main() {
    accel(N, force1);  /* with N points and force= 4*cos(t-3); */
    accel(N, force2);  /* with N points and force= 4*(t*t-3); */
    return 0;
}
