/* File: accelstruct.cpp
   Use a structure to calculate and plot accelerations */
#include <stdio.h> 
#include <math.h>     /* for sin() */   
#include <chplot.h>   /* for plotxy() */
#include "accel.h"    /* local header file */

double force(double t) {  /* function to calculate the force */
    double p;

    p = 4*(sin(t-3))+20;
    return p;
}

double accel(const struct Body *b, double t) { /* for acceleration */
    double a, p;

    p = force(t);
    a = (p-b->mu*b->m*M_G)/b->m;
    return a;
}

/* plot accelerations */
void plotAccel(const struct Body *b, double t0, double tf) { 
    double a[N], t[N];   /* define arrays for plotting */
    int i;

    for(i=0; i<N; i++) {
      t[i] = t0 + i*(t0-tf)/(N-1); 
      a[i] = accel(b, t[i]);
    }
    plotxy(t, a, N, "Acceleration Plot", "time (s)", "accel (m/s^2)");
}


int main() {
    /* declare accel, force, time, initial time, final time, mu, mass */
    double a, p, t, t0, tf, mu, m;
    struct Body b;     /* declare a body with mu and m */

    t = 2.0;           /* set time at 2 seconds */
    t0 = 0.0;          /* initial time 0 second */
    tf = 10.0;         /* final time 10 seconds */
    b.mu = 0.2;        /* mu */
    b.m = 5.0;         /* mass */
    p = force(t);      /* calculate force */
    a = accel(&b, t);  /* calculate acceleration by passing address of b */
    printf("External force p = %f (N)\n", p);
    printf("Acceleration a = %f (m/s^2)\n", a);
    plotAccel(&b, t0, tf); /* plot accelerations by passing address of b */
    return 0;
}
