/************************************************************
* File: student.h 
* This header file contains macros, data structures and 
* function prototypes for the GPA library. Macro names with 
* prefix GPA_,  variable names with prefix gpa_, symbols 
* Birthday, Student, student_t are reserved for this lib.
************************************************************/
#ifndef STUDENT_H
#define STUDENT_H

/* define End Of Data */
#define GPA_EOD -1

/* define boolean numbers */
#define TRUE 1
#define FALSE 0

struct Birthday {             /* structure for birthday */
    short day, month, year;   /* day, month, year */
};

typedef struct Student {      /* structure for a student */
    int id;                   /* student id */
    char name[32];            /* student name */
    struct Birthday birthday; /* student birthday */
    double gpa;               /* student GPA */
} student_t;

/* function prototypes for a single student */
extern void gpa_printStudent(const student_t *sp);
extern int gpa_getId(const student_t *sp);
extern char *gpa_getName(const student_t *sp);
extern struct Birthday gpa_getBirthday(const student_t *sp);
extern double gpa_getGPA(const student_t *sp);

/* function prototypes for a list of students */
extern int gpa_totnum(const student_t *head);
extern void gpa_printList(const student_t *head);
extern double gpa_averageGPA(const student_t *head);
extern student_t *gpa_highestGPA(const student_t *head);
extern student_t *gpa_lowestGPA(const student_t *head);
extern student_t *gpa_find(const student_t *head, int id, const char *name);
extern int gpa_aboveGPA(const student_t *head, double gpa);
extern void gpa_sortGPA(student_t *head);

/* utility function */
extern const char *gpa_monthString(int month);

/* load code from file studentfunc.c for Ch */
#ifdef _CH_
#pragma importf "studentfunc.c"
#endif

#endif /* STUDENT_H */
