/* File: union2.c
   How a union is typically used in an application. */
#include <stdio.h>

union data {      /* define a union */
    int i;        /* a member of int type */
    float f;      /* a member of float type */
    double d;     /* a member of double type */
};

enum datatype {   /* define an enumerate type */
    inttype,      /* int type */
    floattype,    /* float type */
    doubletype    /* double type */
};

struct tag {      /* define a structure */
    enum datatype dtype; /* this enumerate type associates the data 
                            inside union n. */
    union data u;  /* union contains the data specified by dtype */
};

/* function prototypes */
void storeData(enum datatype, struct tag *, const void *); /* store data */
void printData(const struct tag *);                        /* print data */

int main() {
    /* declare and initialize variables of different types */
    int i = 10;       /* declare i of int type */
    float f = 20.0;   /* declare f of float type */
    double d = 30.0;  /* declare d of double type */
    struct tag s;     /* declare s of strucutre tag */

    storeData(inttype, &s, &i); /* store i inside s based on inttype */
    printData(&s);    /* print the stored data based on dtype inside s */
    storeData(floattype, &s, &f); /* store f inside s based on inttype */
    printData(&s);    /* print the stored data based on dtype inside s */
    storeData(doubletype, &s, &d); /* store d inside s based on inttype */
    printData(&s);    /* print the stored data based on dtype inside s */
    return 0;
}

/* store data passed by the generic pointer in the memory pointed by sp
   with data type specified by dtype, object pointed by sp WILL BE modified */
void storeData(enum datatype dtype, struct tag *sp, const void *data) {
    sp->dtype = dtype;  /* save the stored data type in sp */
    switch(dtype) {     /* check the data type of the passed data */
      case inttype:            /* int type  */
       /* cast data from the generic pointer into 'int *', 
          obtain the data by *((int*) data), store it in sp->u.i */
        sp->u.i = *(int*)data; 
        break;
      case floattype:           /* do the same for float type */
        sp->u.f = *(float*)data;
        break;
      case doubletype:          /* do the same for double type */
        sp->u.d = *(double*)data;
        break;
      default:                  /* in case a mistake is made */
        printf("Error: unknown data type\n");
        break;
    }
}

/* print the stored data in sp, object pointed by sp 
   WILL NOT BE modified. sp is const-qualified. */
void printData(const struct tag *sp) { 
    switch(dtype) {     /* check the data type of the stored data */
      case inttype:                     /* int type  */
        printf("data = %d\n", sp->u.i); /* print int */
        break;
      case floattype:                   /* float type */
        printf("data = %f\n", sp->u.f); /* print float */
        break;
      case doubletype:                  /* double type */
        printf("data = %f\n", sp->u.d); /* print double */
        break;
      default:                          /* in case a mistake is made */
        printf("Error: unknown data type\n");
        break;
    }
}
