/* File: copyfile3.c
   Copy a file once in an entire file */
#include <stdio.h>    /* for fopen(), fclose(), fread(), fwrite(), feof() */
#include <sys/stat.h> /* for struct stat, stat() */

int copyfile3(const char *inputfile, const char *outputfile) {
    FILE *fp1, *fp2;   /* declare FILE pointers for input and output files */
    struct stat stbuf;  /* declare stbuf for file status structure */

    /* open input file for reading and output file for writing
       in binary mode */
    if((fp1 = fopen(inputfile, "rb")) == NULL)
        return -1; /* if failed to open the file for reading */
    if((fp2 = fopen(outputfile, "wb")) == NULL) {
        fclose(fp1); 
        return -1; /* if failed to open the file for writing */
    }

    if(!stat(inputfile, &stbuf)) { /* if the input file exists */
        /* allocate the memory to hold the contents of the file */
        char *contents = (char *)malloc(sizeof(stbuf.st_size));
        if(contents) {
           fread(contents, stbuf.st_size, 1, fp1);   /* read the entire file */
           fwrite(contents, stbuf.st_size, 1, fp2);  /* write the entire file */
           free(contents);  /* free the allocated memory */
         }
         else {
            printf("Error: not enough memory\n");
            return -1; /* if failed to allocate memory */
         }
    }
    else {
        printf("Error: call stat() failed\n");
    }
    fclose(fp1);         /* close the file stream for the input file */
    fclose(fp2);         /* close the file stream for the output file */
    return 0;
}
