/* File: recreaddir.c 
   Read all entries in a directory recursively */
#include <dirent.h>     /* for DIR, struct dirent, 
                           opendir(), readdir(), closedir() */
#include <sys/stat.h>   /* for struct stat, stat() */

/* open, read a directory, and go into its subdirectories recursively */ 
void dirwalk(const char *dirname) {
    struct stat stbuf;   /* declare a variable of a stat structure */
    DIR *dirp;           /* declare dirp of DIR type, a directory stream */  
    struct dirent *direntp; /* directory entry */
    char filename[1024];    /* filename */

    /* open a searchable directory */
    if(!stat(dirname, &stbuf) && S_ISDIR(stbuf.st_mode)) {
        dirp = opendir(dirname);  /* open a directory stream */
        if (dirp == NULL)  /* if dirname cannot be opened, return */
          return;           
    }

    /* read the directory and go into its subdirectories recursively */ 
    while(direntp = readdir(dirp)) { /* read one entry at a time */
      /* filename contains dirname/d_name */
      sprintf(filename, "%s/%s", dirname, direntp->d_name);
      /* obtain the information for filename */
      if(!stat(filename, &stbuf)) {
          /* print the size of file in bytes */
          printf("size of %s is %d bytes\n", filename, stbuf.st_size);  

          /* if the file is a directory, except for "." and ".." */
          if((strcmp(".", direntp->d_name) != 0) \
              && (strcmp("..", direntp->d_name) != 0) \
              && S_ISDIR(stbuf.st_mode)) 
          {
            dirwalk(filename);  /* recursive calling this function */
          }
        }
    }
    closedir(dirp); /* close the directory stream */
}

int main() {
    const char *dirname = ".";  /* the current directory */

    dirwalk(dirname);  /* print entries in the directory dirname recursively */
    return 0;
}

