/* File: quadratic3.c (run in C99, Ch, and C++)
   Compile in gcc, use command 'gcc -std=c99 quadratic3.c -lm' 
   Compile in g++, use command 'g++ quadratic3.c -lm' 
   Compile in Visual C++, use command 'cl /TP quadratic3.c' 
                       or use command 'cl quadratic3.cpp' 

   Solve for two complex roots of the quadratic equation
   x^2-4x+13 = 0  in the complex domain.
   This program is portable and can be compiled in both C99 and C++ */
#include <stdio.h>
#include "ccomplex.h" /* portable header for complex numbers in C99 and C++ */

int main() {
    /* Use double_complex to declare and initialize variables */
    double_complex a = 1.0, b = -4.0, c = 13.0, x1, x2, z;

    x1 = (-b +sqrt(b*b-4.0*a*c))/(2.0*a); /* use math function sqrt() */
    x2 = (-b -sqrt(b*b-4.0*a*c))/(2.0*a);
    printf("x1 = complex(%f, %f)\n", creal(x1), cimag(x1));
    printf("x2 = complex(%f, %f)\n", creal(x2), cimag(x2));

    /* Use I, sin(), creal(), cimag(), cabs(), carg(), and polar() */
    z = 1.0 + I*2.0;               /* construct a complex number */
    z = sin(z);                    /* use mathematical function sin() */
    /* get real part creal() and imaginary part cimag() of complex z */
    printf("sin(1+i2) = complex(%f, %f)\n", creal(z), cimag(z));
    z = 3.0 + I*4.0;               /* construct a complex number */
    /* get real part creal() and imaginary part cimag() of complex z */
    printf("z = complex(%f, %f)\n", creal(z), cimag(z));
    /* get modulus cabs() and phase angle carg() of complex z */
    printf("modulus(z) = %f, phase angle (z) = %f\n", cabs(z), carg(z));
    /* form a complex number in the polar form by polar(r, theta) */
    z = polar(cabs(z), carg(z));   
    printf("z = complex(%f, %f)\n", creal(z), cimag(z));
    return 0;
}
