/* File: response.cpp
   Calculate and display unit-step responses of 
   a second-order system with various damping ratios. 
   Compile in g++, use command 
    g++ response.cpp -I/usr/local/silib/include -L/usr/local/silib/lib -lchplot
   Compile in Visual C++, use command 
    cl response.cpp -IC:/silib/include C:/silib/lib/libchplot.lib
*/
#include <chplot.h>
#include "ccomplex.h" /* portable header for C99 and C++ */

#define N 50          /* number of points for plotting */

double zeta;          /* parameter zeta */

double response(double t) {
    double wn = 1.0, y;
    double_complex per, damp, res;

    /* when zeta > 1,
       srqt(1-zeta*zeta) and acos(zeta) are complex numbers */
    damp = exp(-zeta*wn*t)/sqrt(1.0-zeta*zeta+0.0*I);  
    per = sin(wn*sqrt(1.0-zeta*zeta+0.0*I)*t + cacos(zeta-0.0*I));
    res = damp*per;
    /* get rid of tiny imaginary part in res */
    y = 1.0 - creal(res); 
    return y;
}

int main() {
    double t0=0, tf=13; /* time from 0 to 13 seconds */
    int i;
    class CPlot plot;
    char str[64];       /* string for legend */

    plot.enhanceText(); /* use enhanced text for special symbols */
    plot.label(PLOT_AXIS_X, "{/Symbol w}_n t"); /* for w_n t */
    plot.label(PLOT_AXIS_Y, "y(t)");
    for(i=0, zeta=0.1; zeta <= 2; i++, zeta += 0.4) {  
       plot.func2D(t0, tf, N, response);
       sprintf(str, "{/Symbol z} = %.1f", zeta); /* string "zeta = 0.1" */
       plot.legend(str, i);         /* add legend "zeta = 0.1 */
    }
    plot.plotting();
}
