/* File: zeropole.ch  (run in Ch only)
   Multiply polynomials.
   Find zeros and poles of a transfer function */
#include <stdio.h>
#include <numeric.h> /* for conv() and roots() */

/* define dimensions for arrays */
#define M1 2        /* p1 = s+1 */
#define M2 3        /* p2 = s^2 + 2s + 43.25 */
#define M (M1+M2-1) /* num = (s+1)(s^2 + 2s + 43.25) */
#define MM (M-1)    /* roots for num = (s+1)(s^2 + 2s + 43.25) */
#define N1 3        /* p3 = s^2 + 2s + 82 */
#define N2 3        /* p4 = s^2 + 2s + 101 */
#define N (N1+N2-1) /* den = (s^2 + 2s + 82)(s^2 + 2s + 101) */
#define NN (N-1) /* roots for den = (s^2 + 2s + 82)(s^2 + 2s + 101) */

int main() {
   double p1[M1] = {1, 1};        /* p1 = s+1 */
   double p2[M2] = {1, 2, 43.25}; /* p2 = s^2 + 2s + 43.25 */
   double num[M];                 /* num = p1*p2 */
   complex zeros[MM];             /* zeros for numerator num   */
   double p3[N1] = {1, 2, 82};    /* p3 = s^2 + 2s + 82 */
   double p4[N2] = {1, 2, 101};   /* p4 = s^2 + 2s + 101 */
   double den[N];                 /* den = p3*p4 */
   complex poles[NN];             /* poles for denominator for den */
   int i;

   conv(num,p1,p2);     /* num = (s+1)(s^2 + 2s + 43.25) */

   printf("numerator   = "); 
   for(i=0; i<M; i++) { 
     printf("%8.2f", num[i]); 
   }

   conv(den,p3,p4);     /* den = (s^2 + 2s + 82)(s^2 + 2s + 101) */
   printf("\ndenominator = "); 
   for(i=0; i<N; i++) { 
     printf("%8.2f", den[i]); 
   }

   roots(zeros, num);   /* find roots for num = p1*p2 */
   printf("\nzeros = "); 
   for(i=0; i<MM; i++) { 
     printf("%.2f ", zeros[i]); 
   }

   roots(poles, den);   /* find roots for den = p3*p4 */
   printf("\npoles = "); 
   for(i=0; i<NN; i++) { 
     printf("%.2f ", poles[i]); 
   }
   printf("\n"); 
   return 0;
}
