/* File: accelforce.cpp
   Plot acceleration and force with different units on the same figure
   using axes y1 versus x1, y2 versus x1 */
#include <math.h>       // for sin() 
#include <chplot.h>     // for CPlot
#define M_G   9.81      // for gravitational acceleration constant g
#define N     100       // number of points for plotting

int main() {
    double a[N], p[N], t[N], t0, tf;
    double mu, m;
    int i;
    CPlot plot;


    mu = 0.2;
    m = 5.0;
    t0 = 0.0;
    tf = 10.0;
    for(i=0; i<N; i++) {
        t[i] = t0+i*(tf - t0)/(N-1);
        p[i] = 4*sin(t[i]-3)+20;
        a[i] = (p[i]-mu*m*M_G)/m;
    }
    /* Setup for the entire plot */
    plot.title("force and acceleration");
    plot.label(PLOT_AXIS_X, "time (s)");
    plot.border(PLOT_BORDER_ALL, PLOT_ON); // display all four borders

    /* Setup for the acceleration using axes x1 and y1 */
    plot.label(PLOT_AXIS_Y, "accel (m/s^2)");
    plot.data2DCurve(t, a, N);    
    plot.legend("acceleration", 0);  // set legend for 1st curve 

    /* Setup for the force using axes x1 and y2 */
    plot.label(PLOT_AXIS_Y2, "force (N)");
    plot.data2DCurve(t, p, N);    
    plot.axisRange(PLOT_AXIS_Y2, 15, 25);
    plot.ticsRange(PLOT_AXIS_Y2, 5);
    plot.legend("force", 1);  // set legend for 2nd curve
    plot.axes(1, "x1y2");     // use axes x1 and y2 for 2nd curve 
    plot.tics(PLOT_AXIS_Y2, PLOT_ON); 

    plot.plotting();          
    return 0;
}
