/* File: legend.cpp 
   Use legend for a plot with multiple curves */
#include <math.h>    // for sin(), cos(), M_PI
#include <chplot.h>  // for CPlot

#define N 36

int main() {
    int i;
    double x0, xf, x[N], y1[N], y2[N];
    char *title="Sine and Cosine Waves",        
         *xlabel="degree", *ylabel="amplitude";
    CPlot plot;

    x0 = 0;
    xf = 360;
    for(i = 0; i < N; i++) {
        x[i]  = x0 + i*(xf - x0)/(N-1);
        y1[i] = sin(x[i]*M_PI/180);
        y2[i] = cos(x[i]*M_PI/180);
    }
    plot.title(title);
    plot.label(PLOT_AXIS_X, xlabel);
    plot.label(PLOT_AXIS_Y, ylabel);
    plot.data2DCurve(x, y1, N);    
    plot.data2DCurve(x, y2, N);    
    plot.legend("sin(x)", 0);         // add legend for 1st curve
    plot.legend("cos(x)", 1);         // add legend for 2nd curve
    plot.legendLocation(260, 0.6);    // set the location for legends
    plot.axisRange(PLOT_AXIS_X, 0.0, 360.0);       // set axis range
    plot.ticsRange(PLOT_AXIS_X, 60.0, 0.0, 360.0); // set tics marks
    plot.plotting();          
    return 0;
}
