/* File: linreg.ch
   Perform linear regression analysis for a set of data in computational arrays.
   Plot the fitted line in comparison with the original scattered data points */
#include <stdio.h>
#include <numeric.h> /* for 'array' and linsolve() */
#include <chplot.h>  /* for class CPlot for plotting */
#define N     8      /* number of points for sample data */
#define NUM   100    /* number of points for plotting */ 
double b, v;         /* linear regression coefficients */

/* linear function y = b + v*t */
double func(double t) {
    double y;
    y = b + v*t;
    return y;
}

int main() {
    /* declare and initialize the original data */
    array double t[N] = {0, 3,   5, 8,  10,   12,   15,   18};
    array double y[N] = {2, 5.2, 6, 10, 10.4, 13.4, 14.8, 18}, x[2];
    array double a[N][2]; /* declare array a */
    CPlot plot;           /* instantiate a plotting class */
    int i;                /* index variable for a for-loop */

    /* initialize array a based on the matrix formula for linear regression */
    for(i=0; i<N; i++) {
       a[i][0] = 1;
       a[i][1] = t[i];
    }
    /* calculate the linear regression coefficients based on the formulas */
    linsolve(x, transpose(a)*a, transpose(a)*y);
    /* or  x = inverse(transpose(a)*a)*transpose(a)*y; */
    b = x[0];
    v = x[1];
    /* display the linear regresson coefficients v and b, and find y(11.0) */
    printf("v = %.2f\n", v);
    printf("b = %.2f\n", b);
    printf("y is = %.2f (m) at time t = 11 (s)\n", func(11.0));

   /* set the label on the x-axis */
    plot.label(PLOT_AXIS_X, "t (second)");
    /* set the label on the y-axis */
    plot.label(PLOT_AXIS_Y, "y (meter)");
    /* set the range for x-axis from 0 to 18 */
    plot.axisRange(PLOT_AXIS_Y, 0.0, 18.0);
    /* add data y[] and t[] as the 1st set of data for plotting */
    plot.data2DCurve(t, y, N);
    /* set the scattered point plot type for the 1st set of data */
    plot.plotType(PLOT_PLOTTYPE_POINTS, 0);
    /* set function func() to be plotted from t[0] to t[n-1] with NUM points */
    plot.func2D(t[0], t[N-1], NUM, func);
    /* after the above setup, ready and go, plot and display it! */
    plot.plotting();
    return 0;
}
