/* File: acceldata2DCurve2.ch
   Plot accelereration versus time with 11 and 100 points on the same figure */ 
#include <stdio.h>
#include <math.h>       /* for sin() */ 
#include <chplot.h>     /* for class CPlot for plotting */
#include <numeric.h>    /* for lindata() */

#define M_G   9.81      /* gravitational acceleration constant g */
#define N1    11        /* num of points for the 1st curve */
#define N2    100       /* num of points for the 2nd curve */

int main() {
    /* declare two sets of computational arrays for plottting for a versus t */
    array double a1[N1], p1[N1], t1[N1], a2[N2], p2[N2], t2[N2];
    double t0, tf, mu, m, p;
    CPlot plot;         /* instantiate a plotting class */

    /* set mu, mass, initial time, and final time */
    mu = 0.2;
    m = 5.0;
    t0 = 0.0;
    tf = 10.0;

    /* calculate t1 and a1 with N1 points */
    lindata(t0, tf, t1);
    p1 = 4*sin(t1-3)+20;
    a1 = (p1-mu*m*M_G)/m;

    /* calculate t2 and a2 with N2 points */
    lindata(t0, tf, t2);
    p2 = 4*sin(t2-3)+20;
    a2 = (p2-mu*m*M_G)/m;

    /* use a plotting class plot two curves on the same figure */
    plot.title("Acceleration Plot");
    plot.label(PLOT_AXIS_X, "time (s)");
    plot.label(PLOT_AXIS_Y, "accel (m/s^2)");
    plot.data2DCurve(t1, a1, N1);
    plot.data2DCurve(t2, a2, N2);
    plot.legend("11 points", 0);
    plot.legend("100 points", 1);
    plot.plotting();
    return 0;
}
